@@ -187,7 +187,7 @@ filename_expand (char *fullpath, char *filename)
 #endif
 
   int len;
-  fullpath[0] = '\0';
+  fullpath[0] = fullpath[QXE_PATH_MAX] = '\0';
 
 #ifdef  CYGWIN
   /*
@@ -200,7 +200,7 @@ filename_expand (char *fullpath, char *filename)
   if (filename[0] && filename[0] == '/')
      {
        /* Absolute (unix-style) pathname.  Do nothing */
-       strcat (fullpath, filename);
+       strncat (fullpath, filename, QXE_PATH_MAX);
      }
   else
     {
@@ -208,15 +208,18 @@ filename_expand (char *fullpath, char *filename)
        and prepend it.  FIXME: need to fix the case of DOS paths like
        "\foo", where we need to get the current drive. */
 
-      strcat (fullpath, get_current_working_directory ());
+      strncat (fullpath, get_current_working_directory (), QXE_PATH_MAX);
       len = strlen (fullpath);
 
-      if (len > 0 && fullpath[len-1] == '/')	/* trailing slash already? */
-	;					/* yep */
-      else
-	strcat (fullpath, "/");		/* nope, append trailing slash */
+      /* If no trailing slash, add one */
+      if (len <= 0 || (fullpath[len - 1] != '/' && len < QXE_PATH_MAX))
+	{
+	  strcat (fullpath, "/");
+	  len++;
+	}
+
       /* Don't forget to add the filename! */
-      strcat (fullpath,filename);
+      strncat (fullpath, filename, QXE_PATH_MAX - len);
     }
 } /* filename_expand */
 
@@ -435,7 +438,7 @@ main (int argc, char *argv[])
 		  break;
 		case 'r':
 		  GET_ARGUMENT (remotearg, "-r");
-		  strcpy (remotepath, remotearg);
+		  strncpy (remotepath, remotearg, QXE_PATH_MAX);
 		  rflg = 1;
 		  break;
 #endif /* INTERNET_DOMAIN_SOCKETS */
@@ -590,7 +593,7 @@ main (int argc, char *argv[])
 					 * to this machine */
 	      if ((ptr = getenv ("GNU_NODE")) != NULL)
 		/* user specified a path */
-		strcpy (remotepath, ptr);
+		strncpy (remotepath, ptr, QXE_PATH_MAX);
 	    }
 #if 0  /* This is really bogus... re-enable it if you must have it! */
 #if defined (hp9000s300) || defined (hp9000s800)
