@@ -1,6 +1,7 @@
 ;;; abbrev.el --- abbrev mode commands for Emacs
 
 ;; Copyright (C) 1985, 1986, 1987, 1992, 1997 Free Software Foundation, Inc.
+;; Copyright (C) 2002 Ben Wing.
 
 ;; Maintainer: XEmacs Development Team
 ;; Keywords: abbrev, dumped
@@ -37,17 +38,28 @@
   :tag "Abbreviations"
   :group 'editing)
 
-(defgroup abbrev-mode nil
+(defcustom abbrev-mode nil
   "Word abbreviations mode."
+  :initialize #'set-default
+  :set #'(lambda (sym val)
+	   (global-abbrev-mode (if val 1 0)))
   :group 'abbrev)
 
-;jwz: this is preloaded so don't ;;;###autoload
-(defcustom only-global-abbrevs nil "\
-*Non-nil means user plans to use global abbrevs only.
+(defcustom only-global-abbrevs nil
+  "*Non-nil means user plans to use global abbrevs only.
 Makes the commands to define mode-specific abbrevs define global ones instead."
   :type 'boolean
   :group 'abbrev)
 
+(defcustom defining-abbrev-turns-on-abbrev-mode t
+  "*NOn-nil turns on abbrev-mode whenever an abbrev is defined.
+This occurs only when the user-level commands (e.g. `add-global-abbrev')
+are used.  abbrev-mode is turned on in all buffers or the current-buffer
+only, depending on whether a global or mode-specific abbrev is defined."
+  ;;#### should turn on in all buffers of current mode in mode-specific abbrev!
+  :type 'boolean
+  :group 'abbrev)
+
 ;;; XEmacs: the following block of code is not in FSF
 (defvar abbrev-table-name-list '()
   "List of symbols whose values are abbrev tables.")
@@ -66,7 +78,6 @@ This causes `save-some-buffers' to offer to save the abbrevs.")
   (setq abbrevs-changed t)
   nil)
 
-
 (defun define-abbrev-table (table-name definitions)
   "Define TABLE-NAME (a symbol) as an abbrev table name.
 Define abbrevs in it according to DEFINITIONS, which is a list of elements
@@ -76,7 +87,9 @@ of the form (ABBREVNAME EXPANSION HOOK USECOUNT)."
           ((not table)
            (setq table (make-abbrev-table))
            (set table-name table)
-           (setq abbrev-table-name-list (cons table-name abbrev-table-name-list)))
+           (setq abbrev-table-name-list
+		 (sort (cons table-name abbrev-table-name-list)
+		       #'string-lessp)))
           (t
            (setq table (wrong-type-argument 'vectorp table))
            (set table-name table)))
@@ -88,15 +101,9 @@ of the form (ABBREVNAME EXPANSION HOOK USECOUNT)."
   "Define an abbrev in TABLE named NAME, to expand to EXPANSION or call HOOK.
 NAME and EXPANSION are strings.  Hook is a function or `nil'.
 To undefine an abbrev, define it with an expansion of `nil'."
-  (unless (or (null expansion) (stringp expansion))
-    (setq expansion (wrong-type-argument 'stringp expansion)))
-
-  (unless (or (null count) (integerp count))
-    (setq count (wrong-type-argument 'fixnump count)))
-
-  (unless (vectorp table)
-    (setq table (wrong-type-argument 'vectorp table)))
-
+  (check-type expansion (or null string))
+  (check-type count (or null integer))
+  (check-type table vector)
   (let* ((sym (intern name table))
          (oexp (and (boundp sym) (symbol-value sym)))
          (ohook (and (fboundp sym) (symbol-function sym))))
@@ -204,64 +211,64 @@ is not undone."
           (goto-char opoint)))))
 
 
-
-(defun insert-abbrev-table-description (name &optional human-readable)
-  "Insert before point a full description of abbrev table named NAME.
-NAME is a symbol whose value is an abbrev table.
-If optional second argument HUMAN-READABLE is non-nil, insert a
-human-readable description. Otherwise the description is an
-expression, a call to `define-abbrev-table', which would define the
-abbrev table NAME exactly as it is currently defined."
-  (let ((table (symbol-value name))
-        (stream (current-buffer)))
-    (message "Abbrev-table %s..." name)
-    (if human-readable
-        (progn
-          (prin1 (list name) stream)
-          ;; Need two terpri's or cretinous edit-abbrevs blows out
-          (terpri stream)
-          (terpri stream)
-          (mapatoms (function (lambda (sym)
-                      (if (symbol-value sym)
-                          (let* ((n (prin1-to-string (symbol-name sym)))
-                                 (pos (length n)))
-                            (princ n stream)
-                            (while (< pos 14)
-                              (write-char ?\  stream)
-                              (setq pos (1+ pos)))
-                            (princ (format " %-5S " (symbol-plist sym))
-                                   stream)
-                            (if (not (symbol-function sym))
-                                (prin1 (symbol-value sym) stream)
-                              (progn
-                                (setq n (prin1-to-string (symbol-value sym))
-                                      pos (+ pos 6 (length n)))
-                                (princ n stream)
-                                (while (< pos 45)
-                                  (write-char ?\  stream)
-                                  (setq pos (1+ pos)))
-                                (prin1 (symbol-function sym) stream)))
-                            (terpri stream)))))
-                    table)
-          (terpri stream))
-        (progn
-          (princ "\(define-abbrev-table '" stream)
-          (prin1 name stream)
-          (princ " '\(\n" stream)
-          (mapatoms (function (lambda (sym)
-                      (if (symbol-value sym)
-                          (progn
-                            (princ "    " stream)
-                            (prin1 (list (symbol-name sym)
-                                         (symbol-value sym)
-                                         (symbol-function sym)
-                                         (symbol-plist sym))
-                                   stream)
-                            (terpri stream)))))
-                    table)
-          (princ "    \)\)\n" stream)))
-    (terpri stream))
-  (message ""))
+; APA: Moved to c (ported function from GNU Emacs to src/abbrev.c)
+; (defun insert-abbrev-table-description (name &optional human-readable)
+;   "Insert before point a full description of abbrev table named NAME.
+; NAME is a symbol whose value is an abbrev table.
+; If optional second argument HUMAN-READABLE is non-nil, insert a
+; human-readable description. Otherwise the description is an
+; expression, a call to `define-abbrev-table', which would define the
+; abbrev table NAME exactly as it is currently defined."
+;   (let ((table (symbol-value name))
+;         (stream (current-buffer)))
+;     (message "Abbrev-table %s..." name)
+;     (if human-readable
+;         (progn
+;           (prin1 (list name) stream)
+;           ;; Need two terpri's or cretinous edit-abbrevs blows out
+;           (terpri stream)
+;           (terpri stream)
+;           (mapatoms (function (lambda (sym)
+;                       (if (symbol-value sym)
+;                           (let* ((n (prin1-to-string (symbol-name sym)))
+;                                  (pos (length n)))
+;                             (princ n stream)
+;                             (while (< pos 14)
+;                               (write-char ?\  stream)
+;                               (setq pos (1+ pos)))
+;                             (princ (format " %-5S " (symbol-plist sym))
+;                                    stream)
+;                             (if (not (symbol-function sym))
+;                                 (prin1 (symbol-value sym) stream)
+;                               (progn
+;                                 (setq n (prin1-to-string (symbol-value sym))
+;                                       pos (+ pos 6 (length n)))
+;                                 (princ n stream)
+;                                 (while (< pos 45)
+;                                   (write-char ?\  stream)
+;                                   (setq pos (1+ pos)))
+;                                 (prin1 (symbol-function sym) stream)))
+;                             (terpri stream)))))
+;                     table)
+;           (terpri stream))
+;         (progn
+;           (princ "\(define-abbrev-table '" stream)
+;           (prin1 name stream)
+;           (princ " '\(\n" stream)
+;           (mapatoms (function (lambda (sym)
+;                       (if (symbol-value sym)
+;                           (progn
+;                             (princ "    " stream)
+;                             (prin1 (list (symbol-name sym)
+;                                          (symbol-value sym)
+;                                          (symbol-function sym)
+;                                          (symbol-plist sym))
+;                                    stream)
+;                             (terpri stream)))))
+;                     table)
+;           (princ "    \)\)\n" stream)))
+;     (terpri stream))
+;   (message ""))
 ;;; End code not in FSF
 
 (defun abbrev-mode (arg)
@@ -276,6 +283,22 @@ and be replaced by its expansion."
   ;; XEmacs change
   (redraw-modeline))
 
+;; XEmacs change
+(defun global-abbrev-mode (arg)
+  "Toggle abbrev mode in all buffers.
+With argument ARG, enable abbrev mode globally if ARG is positive, else
+disable.  In abbrev mode, inserting an abbreviation causes it to expand
+and be replaced by its expansion."
+  (interactive "P")
+  (let ((newval (if (null arg) (not abbrev-mode)
+		  (> (prefix-numeric-value arg) 0))))
+    (setq-default abbrev-mode newval)
+    (loop for buf being the buffers do
+      (if (not (eq (symbol-value-in-buffer 'abbrev-mode buf) newval))
+	  (with-current-buffer buf
+	    (setq abbrev-mode newval)))))
+  (redraw-modeline))
+
 
 (defvar edit-abbrevs-map nil
   "Keymap used in edit-abbrevs.")
@@ -426,14 +449,15 @@ ARG is the argument to `add-global-abbrev' or `add-mode-abbrev'."
        (buffer-substring
 	(point)
 	(if (= arg 0) (mark)
-	  (save-excursion (forward-word (- arg)) (point))))))
+	  (save-excursion (backward-word arg) (point))))))
 
 (defun add-mode-abbrev (arg)
-  "Define mode-specific abbrev for last word(s) before point.
-Argument is how many words before point form the expansion;
-or zero means the region is the expansion.
-A negative argument means to undefine the specified abbrev.
-Reads the abbreviation in the minibuffer.
+  "Define mode-specific abbrev for region or last word(s) before point.
+If region active, use it as the expansion\; otherwise, choose the word
+before point.  A prefix argument specifies the number of words before point
+that form the expansion; or zero means the text between point and mark is
+the expansion.  A negative argument means to undefine the specified abbrev.
+This command uses the minibuffer to read the abbreviation.
 
 Don't use this function in a Lisp program; use `define-abbrev' instead."
   ;; XEmacs change:
@@ -443,19 +467,22 @@ Don't use this function in a Lisp program; use `define-abbrev' instead."
        global-abbrev-table
      (or local-abbrev-table
 	 (error "No per-mode abbrev table")))
-   "Mode" arg))
+   "Mode" arg)
+  (if defining-abbrev-turns-on-abbrev-mode (abbrev-mode 1)))
 
 (defun add-global-abbrev (arg)
-  "Define global (all modes) abbrev for last word(s) before point.
-The prefix argument specifies the number of words before point that form the
-expansion; or zero means the region is the expansion.
-A negative argument means to undefine the specified abbrev.
+  "Define global (all modes) abbrev for region or last word(s) before point.
+If region active, use it as the expansion\; otherwise, choose the word
+before point.  A prefix argument specifies the number of words before point
+that form the expansion; or zero means the text between point and mark is
+the expansion.  A negative argument means to undefine the specified abbrev.
 This command uses the minibuffer to read the abbreviation.
 
 Don't use this function in a Lisp program; use `define-abbrev' instead."
   ;; XEmacs change:
   (interactive "P")
-  (add-abbrev global-abbrev-table "Global" arg))
+  (add-abbrev global-abbrev-table "Global" arg)
+  (if defining-abbrev-turns-on-abbrev-mode (global-abbrev-mode 1)))
 
 (defun add-abbrev (table type arg)
   "Add an abbreviation to abbrev table TABLE.
@@ -484,7 +511,7 @@ the user declines to confirm redefining an existing abbrev."
 ARG is the argument to `inverse-add-global-abbrev' or
 `inverse-add-mode-abbrev'."
   (save-excursion
-    (forward-word (- arg))
+    (backward-word arg)
     (buffer-substring (point) (progn (forward-word 1) (point)))))
 
 (defun inverse-add-mode-abbrev (arg)
@@ -498,7 +525,8 @@ Expands the abbreviation after defining it."
        global-abbrev-table
      (or local-abbrev-table
 	 (error "No per-mode abbrev table")))
-   "Mode" arg))
+   "Mode" arg)
+  (if defining-abbrev-turns-on-abbrev-mode (abbrev-mode 1)))
 
 (defun inverse-add-global-abbrev (arg)
   "Define last word before point as a global (mode-independent) abbrev.
@@ -506,12 +534,13 @@ With prefix argument N, defines the Nth word before point.
 This command uses the minibuffer to read the expansion.
 Expands the abbreviation after defining it."
   (interactive "p")
-  (inverse-add-abbrev global-abbrev-table "Global" arg))
+  (inverse-add-abbrev global-abbrev-table "Global" arg)
+  (if defining-abbrev-turns-on-abbrev-mode (global-abbrev-mode 1)))
 
 (defun inverse-add-abbrev (table type arg)
   (let (name nameloc exp)
     (save-excursion
-     (forward-word (- arg))
+     (backward-word arg)
      (setq name (buffer-substring (point) (progn (forward-word 1)
 					       (setq nameloc (point))))))
     (set-text-properties 0 (length name) nil name)
@@ -554,7 +583,7 @@ If called from a Lisp program, arguments are START END &optional NOQUERY."
 	(if (abbrev-expansion
 	     (setq string
 		   (buffer-substring
-		    (save-excursion (forward-word -1) (point))
+		    (save-excursion (backward-word) (point))
 		    pnt)))
 	    (if (or noquery (y-or-n-p (format "Expand `%s'? " string)))
 		(expand-abbrev)))))))
