@@ -72,6 +72,7 @@ in dumped-lisp.el and is not itself listed.")
        "undo-stack"
        "window"		; simple needs `save-window-excursion'
        "window-xemacs"
+       "resize-minibuffer"	; simple needs `resize-minibuffer-mode'
        "simple"
        "newcomment"
        "keydefs"		; Before loaddefs so that keymap vars exist.
@@ -205,7 +206,6 @@ in dumped-lisp.el and is not itself listed.")
 	   "mule/cyrillic"
 	   "mule/english"
 	   "mule/ethiopic"
-	   "mule/european"
 	   "mule/greek"
 	   "mule/hebrew"
 	   "mule/indian"
@@ -239,6 +239,9 @@ in dumped-lisp.el and is not itself listed.")
        (when (and (featurep 'mule) (valid-console-type-p 'mswindows))
 	 "mule/mule-msw-init-late")
 
+       (when (featurep 'mule)
+	 "mule/general-late")
+
 ;;; mule-load.el ends here
 
 ;; preload InfoDock stuff.  should almost certainly not be here if
@@ -286,7 +289,6 @@ in dumped-lisp.el and is not itself listed.")
        "fontl-hooks"
        "auto-show"
        "paragraphs"             ; needs easy-mmode, coding
-       "resize-minibuffer"
        (when (featurep 'ldap) "ldap")
 
 ;; (when (featurep 'energize) "energize/energize-load.el")
