@@ -153,7 +153,7 @@ Optional arguments are a list of Emacs Lisp directories to compile from; no
 arguments compiles from `load-path'."
   (save-excursion
     ;; XEmacs change
-    (find-file "finder-inf.el")
+    (find-file (expand-file-name "finder-inf.el" lisp-directory))
     (let ((processed nil)
 	  (directory-abbrev-alist
 	   (append
@@ -288,7 +288,7 @@ arguments compiles from `load-path'."
 	  found)
       (while (and dirs (not found))
 	(if (file-exists-p (expand-file-name (concat file ".el") (car dirs)))
-	    (setq found (expand-file-name file (car dirs)))
+	    (setq found (expand-file-name (concat file ".el") (car dirs)))
 	  (if (file-exists-p (expand-file-name file (car dirs)))
 	      (setq found (expand-file-name file (car dirs)))))
 	(setq dirs (cdr dirs)))
