@@ -183,7 +183,7 @@ Keymap for characters following C-c.")
 
 ;; FSFmacs keyboard.c
 
-(define-key global-tty-map "\C-z" 'suspend-emacs)
+(define-key global-tty-map "\C-z" 'suspend-or-iconify-emacs)
 (define-key global-window-system-map "\C-z" 'zap-up-to-char)
 (define-key global-window-system-map '(control Z) 'iconify-frame)
 (define-key global-map "\C-x\C-z" 'suspend-or-iconify-emacs)
@@ -665,13 +665,13 @@ Keymap for characters following C-c.")
 (define-key global-map 'kp-8		'self-insert-command)
 (define-key global-map 'kp-9		'self-insert-command)
 
-(define-key global-map 'select		'function-key-error)
-(define-key global-map 'print		'function-key-error)
+;(define-key global-map 'select		'function-key-error)
+;(define-key global-map 'print		'function-key-error)
 (define-key global-map 'execute		'execute-extended-command)
-(define-key global-map 'clearline	'function-key-error)
+;(define-key global-map 'clearline	'function-key-error)
 (define-key global-map 'insertline	'open-line)
 (define-key global-map 'deleteline	'kill-line)
-(define-key global-map 'insertchar	'function-key-error)
+;(define-key global-map 'insertchar	'function-key-error)
 (define-key global-map 'deletechar	'delete-char)
 
 ;;; keydefs.el ends here
