@@ -29,35 +29,90 @@
 ;; The character set ISO8859-5 is supported.  KOI-8 and ALTERNATIVNYJ are
 ;; converted to ISO8859-5 internally.
 
-;; Windows-1251 support deleted because XEmacs has automatic support.
+;; [Windows-1251 support deleted because XEmacs has automatic support.]
+
+;; #### We only have automatic support on Windows; that needs to be put
+;; back. Also, the Russian Wikipedia articles on KOI-8 list several other
+;; related encodings--KOI8-U (Ukrainian), KOI8-RU (simultaneous support for
+;; Russian, Belorussian, and Ukrainian), KOI8-C (for languages of the
+;; Caucasus), KOI8-O (Old Church Slavonic)--and it would be nice to have
+;; them. Beyond that, we're currently trashing lots of code points with
+;; KOI-8 R; it would be nice to leverage the Unicode support to not do that. 
 
 ;;; Code:
 
-;; Cyrillic syntax
-(modify-syntax-entry 'cyrillic-iso8859-5 "w")
-(modify-syntax-entry ?,L-(B ".")
-(modify-syntax-entry ?,Lp(B ".")
-(modify-syntax-entry ?,L}(B ".")
+;; Case table:
+(loop
+  for (upper lower)
+  in '((#xcf #xef) ; YA
+       (#xce #xee) ; YU
+       (#xcd #xed) ; E
+       (#xcc #xec) ; SOFT SIGN
+       (#xcb #xeb) ; YERU
+       (#xca #xea) ; HARD SIGN
+       (#xc9 #xe9) ; SHCHA
+       (#xc8 #xe8) ; SHA
+       (#xc7 #xe7) ; CHE
+       (#xc6 #xe6) ; TSE
+       (#xc5 #xe5) ; HA
+       (#xc4 #xe4) ; EF
+       (#xc3 #xe3) ; U
+       (#xc2 #xe2) ; TE
+       (#xc1 #xe1) ; ES
+       (#xc0 #xe0) ; ER
+       (#xbf #xdf) ; PE
+       (#xbe #xde) ; O
+       (#xbd #xdd) ; EN
+       (#xbc #xdc) ; EM
+       (#xbb #xdb) ; EL
+       (#xba #xda) ; KA
+       (#xb9 #xd9) ; SHORT I
+       (#xb8 #xd8) ; I
+       (#xb7 #xd7) ; ZE
+       (#xb6 #xd6) ; ZHE
+       (#xb5 #xd5) ; IE
+       (#xb4 #xd4) ; DE
+       (#xb3 #xd3) ; GHE
+       (#xb2 #xd2) ; VE
+       (#xb1 #xd1) ; BE
+       (#xb0 #xd0) ; A
+       (#xaf #xff) ; DZHE
+       (#xae #xfe) ; SHORT U
+       (#xac #xfc) ; KJE
+       (#xab #xfb) ; TSHE
+       (#xaa #xfa) ; NJE
+       (#xa9 #xf9) ; LJE
+       (#xa8 #xf8) ; JE
+       (#xa7 #xf7) ; YI
+       (#xa6 #xf6) ; BYELORUSSIAN-UKRAINIAN I
+       (#xa5 #xf5) ; DZE
+       (#xa4 #xf4) ; UKRAINIAN IE
+       (#xa3 #xf3) ; GJE
+       (#xa2 #xf2) ; DJE
+       (#xa1 #xf1)) ; IO
+  with case-table = (standard-case-table)
+  do
+  (put-case-table-pair (make-char 'cyrillic-iso8859-5 upper)
+		       (make-char 'cyrillic-iso8859-5 lower)
+		       case-table))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; CYRILLIC
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-;; ISO-8859-5
-
-; (make-charset 'cyrillic-iso8859-5 
-; 	      "Right-Hand Part of Latin/Cyrillic Alphabet (ISO/IEC 8859-5): ISO-IR-144"
-; 	      '(dimension
-; 		1
-; 		registry "ISO8859-5"
-; 		chars 96
-; 		columns 1
-; 		direction l2r
-; 		final ?L
-; 		graphic 1
-; 		short-name "RHP of ISO8859/5"
-; 		long-name "RHP of Cyrillic (ISO 8859-5): ISO-IR-144"
-; 		))
+;; The default character syntax is now word. Pay attention to the
+;; exceptions in ISO-8859-5, copying them from ISO-8859-1. 
+(loop
+  for (latin-1 cyrillic) 
+  in '((#xAD #xAD)  ;; SOFT HYPHEN
+       (#xA7 #xFD)  ;; SECTION SIGN
+       (#xA0 #xA0)) ;; NO BREAK SPACE
+  with syntax-table = (standard-syntax-table)
+  do (modify-syntax-entry
+      (make-char 'cyrillic-iso8859-5 cyrillic)
+      (string (char-syntax (make-char 'latin-iso8859-1 latin-1)))
+      syntax-table))
+  
+;; Take NUMERO SIGN's syntax from #. 
+(modify-syntax-entry (make-char 'cyrillic-iso8859-5 #xF0)
+                     (string (char-syntax ?\# (standard-syntax-table)))
+                     (standard-syntax-table))
 
 (make-coding-system
  'iso-8859-5 'iso2022
@@ -66,13 +121,13 @@
    charset-g1 cyrillic-iso8859-5
    charset-g2 t
    charset-g3 t
-   mnemonic "ISO8/Cyr"
-   ))
+   mnemonic "ISO8/Cyr"))
 
 (set-language-info-alist
  "Cyrillic-ISO" '((charset cyrillic-iso8859-5)
 		  (tutorial . "TUTORIAL.ru")
 		  (coding-system iso-8859-5)
+		  (native-coding-system iso-8859-5)
 		  (coding-priority iso-8859-5)
 		  (input-method . "cyrillic-yawerty")
 		  (features cyril-util)
@@ -111,12 +166,10 @@
       (let* ((ch (aref cyrillic-koi8-r-decode-table i))
 	     (split (split-char ch)))
 	(cond ((eq (car split) 'cyrillic-iso8859-5)
-	       (aset table (logior (nth 1 split) 128) i)
-	       )
+	       (aset table (logior (nth 1 split) 128) i))
 	      ((eq ch 32))
 	      ((eq (car split) 'ascii)
-	       (aset table ch i)
-	       )))
+	       (aset table ch i))))
       (setq i (1+ i)))
     table)
   "Cyrillic KOI8-R encoding table.")
@@ -176,6 +229,7 @@
 (set-language-info-alist
  "Cyrillic-KOI8" '((charset cyrillic-iso8859-5)
 		   (coding-system koi8-r)
+		   (native-coding-system koi8-r)
 		   (coding-priority koi8-r)
 		   (input-method . "cyrillic-yawerty")
 		   (features cyril-util)
@@ -282,6 +336,7 @@
 (set-language-info-alist
  "Cyrillic-ALT" '((charset cyrillic-iso8859-5)
 		  (coding-system alternativnyj)
+		  (native-coding-system alternativnyj)
 		  (coding-priority alternativnyj)
 		  (input-method . "cyrillic-yawerty")
 		  (features cyril-util)
