@@ -1,6 +1,7 @@
 ;;; english.el --- English support -*- coding: iso-2022-7bit; -*-
 
 ;; Copyright (C) 1997,1999 Electrotechnical Laboratory, JAPAN.
+;; Copyright (C) 2001, 2002 Ben Wing.
 ;; Licensed to the Free Software Foundation.
 
 ;; Keywords: multibyte character, character set, syntax, category
@@ -30,15 +31,28 @@
 
 ;;; Code
 
-(defun setup-english-environment ()
-  "Reset multilingual environment of Emacs to the default status.
-See the function `reset-language-environment' for more detail."
-  (interactive)
-  (reset-language-environment))
+;; ASCII with right-to-left direction.
+(make-charset 'ascii-right-to-left 
+	      "ASCII (left half of ISO 8859-1) with right-to-left direction"
+	      '(dimension
+		1
+		registries ["ISO8859-1"]
+		chars 94
+		columns 1
+		direction r2l
+		final ?B
+		graphic 0
+		short-name "rev ASCII"
+		long-name "ASCII with right-to-left direction"
+		))
 
 (set-language-info-alist
  "English" '((tutorial . "TUTORIAL")
+	     (locale "en" "C")
 	     (charset ascii)
+             (coding-system iso-8859-1)
+             (coding-priority iso-8859-1)
+             (native-coding-system iso-8859-1)
 	     (sample-text . "Hello!, Hi!, How are you?")
 	     (documentation . "\
 Nothing special is needed to handle English.")
