@@ -32,7 +32,7 @@
 (make-charset 'ethiopic "Ethiopic characters"
 	      '(dimension
 		2
-		registry "Ethiopic-Unicode"
+		registries ["Ethiopic-Unicode"]
 		chars 94
 		columns 2
 		direction l2r
@@ -78,9 +78,12 @@
 	      (charset ethiopic)
 	      (coding-system iso-2022-7bit)
 	      (coding-priority iso-2022-7bit)
-	      (input-method . "ethiopic")
+	      ;; (input-method . "ethiopic")
 	      (features ethio-util)
 	      (sample-text . "$(3$Q#U!.(B")
 	      (documentation . t)))
 
+;; In a more ideal world, we could set the default face fallback from here
+;; to use one of the misc-fixed sizes that handles Ethiopic.
+
 ;;; ethiopic.el ends here
