@@ -38,42 +38,16 @@
 
 ;;;; Classifying text according to charsets
 
-;; the old version was broken in a couple of ways
-;; this is one of several versions, I tried a hash as well as the
-;; `prev-charset' cache used in the old version, but this was definitely
-;; faster than the hash version and marginally faster than the prev-charset
-;; version
-;; #### this really needs to be moved into C
-(defun charsets-in-region (start end &optional buffer)
-  "Return a list of the charsets in the region between START and END.
-BUFFER defaults to the current buffer if omitted."
-  (let (list)
-    (save-excursion
-      (if buffer
-	  (set-buffer buffer))
-      (save-restriction
-	(narrow-to-region start end)
-	(goto-char (point-min))
-	(while (not (eobp))
-	  ;; the first test will usually succeed on testing the
-	  ;; car of the list; don't waste time let-binding.
-	  (or (memq (char-charset (char-after (point))) list)
-	      (setq list (cons (char-charset (char-after (point))) list)))
-	  (forward-char))))
-    list))
-
 (defun charsets-in-string (string)
   "Return a list of the charsets in STRING."
-  (let (list)
-    (mapc (lambda (ch)
-	    ;; the first test will usually succeed on testing the
-	    ;; car of the list; don't waste time let-binding.
-	    (or (memq (char-charset ch) list)
-		(setq list (cons (char-charset ch) list))))
-	  string)
-    list))
+  (let (res)
+    (with-string-as-buffer-contents string
+      ;; charsets-in-region now in C. 
+      (setq res (charsets-in-region (point-min) (point-max))))
+    res))
 
 (defalias 'find-charset-string 'charsets-in-string)
+
 (defalias 'find-charset-region 'charsets-in-region)
 
 
@@ -106,12 +80,33 @@ Only left-to-right is currently implemented."
       0
     1))
 
-;; Not in Emacs/Mule
+;; Not in GNU Emacs/Mule
 (defun charset-registry (charset)
-  "Return the registry of CHARSET.
-This is a regular expression matching the registry field of fonts
-that can display the characters in CHARSET."
-  (charset-property charset 'registry))
+  "Obsolete; use charset-registries instead. "
+  (lwarn 'xintl 'warning 
+    "charset-registry is obsolete--use charset-registries instead. ")
+  (when (charset-property charset 'registries)
+    (elt (charset-property charset 'registries) 0)))
+
+(make-obsolete 'charset-registry 'charset-registries)
+
+(defun charset-registries (charset)
+  "Return the registries of CHARSET."
+  (charset-property charset 'registries))
+
+(defun set-charset-registry (charset registry)
+  "Obsolete; use set-charset-registries instead. "
+  (check-argument-type 'stringp registry)
+  (check-argument-type 'charsetp (find-charset charset))
+  (unless (equal registry (regexp-quote registry))
+    (lwarn 'xintl 'warning
+      "Regexps no longer allowed for charset-registry. Treating %s%s"
+      registry " as a string."))
+  (set-charset-registries 
+   charset 
+   (apply 'vector registry (append (charset-registries charset) nil))))
+
+(make-obsolete 'set-charset-registry 'set-charset-registries)
 
 (defun charset-ccl-program (charset)
   "Return the CCL program of CHARSET.
@@ -131,6 +126,7 @@ See `make-charset'."
 
 (defsetf charset-registry    set-charset-registry)
 (defsetf charset-ccl-program set-charset-ccl-program)
+(defsetf charset-registries  set-charset-registries)
 
 ;;; FSF compatibility functions
 (defun charset-after (&optional pos)
