@@ -34,6 +34,17 @@
 
 ;;; Code:
 
+(make-charset 'thai-xtis "Precomposed Thai (XTIS by Virach)."
+	      '(registries ["xtis-0"]
+		dimension 2
+		columns 1
+		chars 94
+		final ??
+		graphic 0))
+
+(define-category ?x "Precomposed Thai character.")
+(modify-category-entry 'thai-xtis ?x)
+
 (when (featurep 'xemacs)
   (let ((deflist	'(;; chars	syntax
 			  ("$(?!0(B-$(?NxP0R0S0`0(B-$(?e0(B"	"w")
@@ -340,10 +351,11 @@
     (progn
       (make-coding-system
        'tis-620 'ccl
-       "external=tis620, internal=thai-xtis"
+       "TIS620 (Thai)"
        `(mnemonic "TIS620"
-		  decode ,ccl-decode-thai-xtis
-		  encode ,ccl-encode-thai-xtis))
+	 decode ccl-decode-thai-xtis
+	 encode ccl-encode-thai-xtis
+	 documentation "external=tis620, internal=thai-xtis"))
       (coding-system-put 'tis-620 'category 'iso-8-1))
   (make-coding-system
    'tis-620 4 ?T "external=tis620, internal=thai-xtis"
