@@ -2,12 +2,15 @@
 
 Copyright (C) 2000 Andy Piper. */
 
-#define XEMACS_INFO_XEMACS_ORG_REGISTRY_NAME "GNU"
+#define XEMACS_INFO_XEMACS_ORG_REGISTRY_NAME "XEmacs"
 #define XEMACS_INFO_XEMACS_REGISTRY_NAME "XEmacs"
 /* Keys */
 #define XEMACS_INFO_XEMACS_ROOT_KEY "emacs_dir"
 #define XEMACS_INFO_XEMACS_ROOT_TYPE "install_type"
 #define XEMACS_INFO_XEMACS_PACKAGE_KEY "EMACSPACKAGEPATH"
+#define XEMACS_INFO_XEMACS_EARLY_PACKAGES_KEY "EMACSEARLYPACKAGES"
+#define XEMACS_INFO_XEMACS_LATE_PACKAGES_KEY "EMACSLATEPACKAGES"
+#define XEMACS_INFO_XEMACS_LAST_PACKAGES_KEY "EMACSLASTPACKAGES"
 #define XEMACS_INFO_XEMACS_VERSION_KEY "version"
 
 #define XEMACS_DEFAULT_ROOT "\\Program Files\\XEmacs"
@@ -20,8 +23,8 @@ Copyright (C) 2000 Andy Piper. */
 #define XEMACS_PACKAGE_DIR \
   ((char*)(install_type == IDC_INSTALL_NATIVE ? "\\xemacs-packages\\" \
 				    : "/lib/xemacs/xemacs-packages/"))
-#define XEMACS_NATIVE_ARCH_NAME "i386-pc-win32"
-#define XEMACS_CYGWIN_ARCH_NAME "i686-pc-cygwin32"
+#define XEMACS_NATIVE_ARCH_NAME "i586-pc-win32"
+#define XEMACS_CYGWIN_ARCH_NAME "i686-pc-cygwin"
 
 /*
  * Installation hierarchy is:
@@ -36,7 +39,7 @@ Copyright (C) 2000 Andy Piper. */
  * 			\site-packages
  *			\mule-packages
  *			\XEmacs-21.4
- *				\i386-pc-win32
+ *				\i586-pc-win32
  *				\etc
  *				\info
  *				\lisp
@@ -46,11 +49,11 @@ Copyright (C) 2000 Andy Piper. */
  * 		/xemacs/site-packages
  *		/xemacs/mule-packages
  *		/xemacs-21.4
- *				/i686-pc-cygwin32
+ *				/i686-pc-cygwin
  *				/etc
  *				/info
  *				/lisp
- * /usr/local/bin/i686-pc-cygwin32
+ * /usr/local/bin/i686-pc-cygwin
  */
 
  
