@@ -34,6 +34,9 @@ DECLARE_CONSOLE_TYPE (stream);
 
 struct stream_console
 {
+#ifdef NEW_GC
+  struct lrecord_header header;
+#endif /* NEW_GC */
   FILE *in;
   FILE *out;
   FILE *err;
@@ -41,6 +44,17 @@ struct stream_console
   Lisp_Object instream;
 };
 
+#ifdef NEW_GC
+typedef struct stream_console Lisp_Stream_Console;
+
+DECLARE_LRECORD (stream_console, Lisp_Stream_Console);
+
+#define XSTREAM_CONSOLE(x) \
+  XRECORD (x, stream_console, Lisp_Stream_Console)
+#define wrap_stream_console(p) wrap_record (p, stream_console)
+#define STREAM_CONSOLE_P(x) RECORDP (x, stream_console)
+#endif /* NEW_GC */
+
 #define CONSOLE_STREAM_DATA(con) CONSOLE_TYPE_DATA (con, stream)
 
 #endif /* INCLUDED_console_stream_impl_h_ */
