@@ -29,7 +29,9 @@ Boston, MA 02111-1307, USA.  */
 
 #include "console.h"
 
+#ifndef NEW_GC
 extern const struct sized_memory_description stream_console_data_description;
+#endif /* not NEW_GC */
 
 extern Lisp_Object Vterminal_console, Vterminal_frame, Vterminal_device;
 
