@@ -1863,7 +1863,14 @@ gtk_reset_modifier_mapping (struct device *d)
   xd->lock_interpretation = 0;
 
   if (map)
-    XFreeModifiermap (map);
+    {
+      XFreeModifiermap (xd->x_modifier_keymap);
+      /* Set it to NULL in case we receive two MappingModifier events in a
+         row, and the second is processed during some CHECK_QUITs within
+         x_reset_key_mapping. If that happens, XFreeModifierMap will be
+         called twice on the same map, and we crash.  */
+      xd->x_modifier_keymap = NULL;
+    }
 
   gtk_reset_key_mapping (d);
 
