@@ -745,7 +745,14 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
   Bool y_position_specified_p = False;
   Bool internal_border_width_specified = False;
   Lisp_Object tail;
-  Widget w = FRAME_X_TEXT_WIDGET (f);
+  Widget w;
+
+  /* We can be called after the X IO error handler has seen a broken pipe on
+     the relevant display. Don't do anything in that case.  */
+  if (!FRAME_LIVE_P (f) || DEVICE_X_BEING_DELETED (XDEVICE (FRAME_DEVICE (f))))
+    return;
+
+  w = FRAME_X_TEXT_WIDGET (f);
 
   for (tail = plist; !NILP (tail); tail = Fcdr (Fcdr (tail)))
     {
@@ -776,12 +783,12 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
 			     /* !!#### Verify this + 1 and document
 				as zero-termination */
 			     XtRString, extval, extvallen + 1,
-			     (XtArgVal) NULL);
+			     NULL);
 	    }
 	  else
 	    XtVaSetValues (w, XtVaTypedArg, extprop, XtRInt,
 			   XINT (val), sizeof (int),
-			   (XtArgVal) NULL);
+			   NULL);
 	}
       else if (SYMBOLP (prop))
 	{
@@ -880,7 +887,7 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
 			     /* !!#### Verify this + 1 and document
 				as zero-termination */
 			     XtRString, extval, extvallen + 1,
-			     (XtArgVal) NULL);
+			     NULL);
 	    }
 
 #ifdef HAVE_SCROLLBARS
@@ -2727,6 +2734,17 @@ x_update_frame_external_traits (struct frame *frm, Lisp_Object name)
    {
      Lisp_Object font = FACE_FONT (Vdefault_face, frame, Vcharset_ascii);
 
+     /* It may be that instantiating the font has deleted the frame (will
+	happen if the user has specified a charset registry for ASCII that
+	isn't available on the server, and our fallback of iso8859-1 isn't
+	available; something vanishingly rare.) In that case, return from
+	this function without further manipulation of the dead frame. */
+
+     if (!FRAME_LIVE_P(frm))
+       {
+	 return;
+       }
+
      /* #### what to do about Xft?  I don't think the font is actually used
 	to compute cell size for computing frame pixel dimensions (see call
 	to EmacsFrameRecomputeCellSize() below); where is it used? -- sjt
