@@ -73,6 +73,7 @@ SYMBOL_KEYWORD (Q_callback);
 SYMBOL_KEYWORD (Q_callback_ex);
 SYMBOL (Qcancel);
 SYMBOL (Qcategory);
+SYMBOL (Qccl_program);
 SYMBOL (Qcenter);
 SYMBOL (Qchain);
 SYMBOL (Qchange);
@@ -115,6 +116,7 @@ SYMBOL (Qduplex);
 SYMBOL (Qdynarr_overhead);
 SYMBOL (Qemergency);
 SYMBOL (Qempty);
+SYMBOL (Qencode_as_utf_8);
 SYMBOL (Qeq);
 SYMBOL (Qeql);
 SYMBOL (Qequal);
@@ -128,6 +130,7 @@ SYMBOL (Qfallback);
 SYMBOL (Qfile);
 SYMBOL_MODULE_API (Qfile_name);
 SYMBOL_KEYWORD (Q_filter);
+SYMBOL (Qfinal);
 SYMBOL (Qfixnum);
 SYMBOL (Qfloat);
 SYMBOL (Qfont);
@@ -155,6 +158,7 @@ SYMBOL (Qhorizontal);
 SYMBOL (Qicon);
 SYMBOL (Qid);
 SYMBOL (Qignore);
+SYMBOL (Qinitial);
 SYMBOL (Qimage);
 SYMBOL_KEYWORD (Q_image);
 SYMBOL_KEYWORD (Q_included);
@@ -284,6 +288,8 @@ SYMBOL (Qtype);
 SYMBOL (Qundecided);
 SYMBOL (Qundefined);
 SYMBOL (Qunimplemented);
+SYMBOL (Qunicode_registries);
+SYMBOL (Qunicode_type);
 SYMBOL (Quser_default);
 SYMBOL_KEYWORD (Q_value);
 SYMBOL (Qvalue_assoc);
