@@ -827,7 +827,10 @@ png_error_func (png_structp UNUSED (png_ptr), png_const_charp msg)
 static void
 png_warning_func (png_structp UNUSED (png_ptr), png_const_charp msg)
 {
-  warn_when_safe (Qpng, Qinfo, "%s", msg);
+  DECLARE_EISTRING (eimsg);
+
+  eicpy_ext(eimsg, msg, Qbinary);  
+  warn_when_safe (Qpng, Qinfo, "%s", eidata(eimsg));
 }
 
 struct png_unwind_data
@@ -846,7 +849,13 @@ png_instantiate_unwind (Lisp_Object unwind_obj)
 
   free_opaque_ptr (unwind_obj);
   if (data->png_ptr)
-    png_destroy_read_struct (&(data->png_ptr), &(data->info_ptr), (png_infopp)NULL);
+    {
+      /* ensure we can't get here again */
+      png_structp tmp = data->png_ptr;
+      data->png_ptr = NULL;
+      png_destroy_read_struct (&tmp, &(data->info_ptr), (png_infopp)NULL);
+    }
+
   if (data->instream)
     retry_fclose (data->instream);
 
@@ -871,24 +880,36 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
   png_structp png_ptr;
   png_infop info_ptr;
 
+  xzero (unwind);
+  record_unwind_protect (png_instantiate_unwind, make_opaque_ptr (&unwind));
+
+  if (setjmp (png_err_stct.setjmp_buffer))
+    {
+      /* Something blew up:
+	 just display the error (cleanup happens in the unwind) */
+      signal_image_error_2 ("Error decoding PNG",
+			     build_string(png_err_stct.err_str),
+			     instantiator);
+    }
+
   /* Initialize all PNG structures */
-  png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, (void*)&png_err_stct,
+  png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING,
+				    (void *) &png_err_stct,
 				    png_error_func, png_warning_func);
   if (!png_ptr)
     signal_image_error ("Error obtaining memory for png_read", instantiator);
+  unwind.png_ptr = png_ptr;
+
   info_ptr = png_create_info_struct (png_ptr);
   if (!info_ptr)
     {
+      unwind.png_ptr = NULL;	/* avoid re-calling png_destroy_read_struct
+				   when unwinding */
       png_destroy_read_struct (&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
       signal_image_error ("Error obtaining memory for png_read", instantiator);
     }
-
-  xzero (unwind);
-  unwind.png_ptr = png_ptr;
   unwind.info_ptr = info_ptr;
 
-  record_unwind_protect (png_instantiate_unwind, make_opaque_ptr (&unwind));
-
   /* This code is a mixture of stuff from Ben's GIF/JPEG stuff from
      this file, example.c from the libpng 0.81 distribution, and the
      pngtopnm sources. -WMP-
@@ -897,16 +918,6 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
      and is no longer usable for previous versions. jh
   */
 
-  /* Set the jmp_buf return context for png_error ... if this returns !0, then
-     we ran into a problem somewhere, and need to clean up after ourselves. */
-  if (setjmp (png_err_stct.setjmp_buffer))
-    {
-      /* Something blew up: just display the error (cleanup happens in the unwind) */
-      signal_image_error_2 ("Error decoding PNG",
-			     build_string(png_err_stct.err_str),
-			     instantiator);
-    }
-
   /* Initialize the IO layer and read in header information */
   {
     Lisp_Object data = find_keyword_in_vector (instantiator, Q_data);
@@ -1018,16 +1029,21 @@ png_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
        unobtrusive. */
     {
       int i;
+      DECLARE_EISTRING (key);
+      DECLARE_EISTRING (text);
 
       for (i = 0 ; i < info_ptr->num_text ; i++)
 	{
 	  /* How paranoid do I have to be about no trailing NULLs, and
 	     using (int)info_ptr->text[i].text_length, and strncpy and a temp
 	     string somewhere? */
+          eireset(key);
+          eireset(text);
+          eicpy_ext(key, info_ptr->text[i].key, Qbinary);
+          eicpy_ext(text, info_ptr->text[i].text, Qbinary);
 
 	  warn_when_safe (Qpng, Qinfo, "%s - %s",
-			  info_ptr->text[i].key,
-			  info_ptr->text[i].text);
+			  eidata(key), eidata(text));
 	}
     }
 #endif
@@ -1221,6 +1237,8 @@ tiff_warning_func (const char *module, const char *fmt, ...)
 #else
   char warn_str[1024];
 #endif
+  DECLARE_EISTRING (eimodule);
+  DECLARE_EISTRING (eiwarnstr);
 
   va_start (vargs, fmt);
 #ifdef HAVE_VSNPRINTF
@@ -1229,8 +1247,13 @@ tiff_warning_func (const char *module, const char *fmt, ...)
   vsprintf (warn_str, fmt, vargs);
 #endif
   va_end (vargs);
+
+  eicpy_ext(eimodule, module, Qbinary);  
+  eicpy_ext(eiwarnstr, warn_str, Qbinary);  
+
   warn_when_safe (Qtiff, Qinfo, "%s - %s",
-		  module, warn_str);
+                  eidata(eimodule), 
+                  eidata(eiwarnstr));
 }
 
 static void
