@@ -323,8 +323,8 @@ linux_play_data_or_file(int fd, Binbyte *data,
                device; repeat until all data has been processed */
   rrtn = length;
   do {
-    for (pptr = data; (prtn = parsesndfile(&pptr,(size_t *)&rrtn,&optr)) > 0; )
-      for (cptr = optr; (crtn = sndcnv(&cptr,(size_t *) &prtn,&sptr)) > 0; ) {
+    for (pptr = data; (prtn = parsesndfile(&pptr,&rrtn,&optr)) > 0; )
+      for (cptr = optr; (crtn = sndcnv(&cptr,&prtn,&sptr)) > 0; ) {
 	for (;;) {
 	  if ((wrtn = write(audio_fd,sptr,crtn)) < 0) {
 	    sound_perror("write"); goto END_OF_PLAY; }
