@@ -38,27 +38,15 @@ Boston, MA 02111-1307, USA.  */
 
 /* #define LOAD_AVE_CVT(x) (int) (((double) (x)) * 100.0 / FSCALE) */
 
-/* Define C_ALLOCA if this machine does not support a true alloca
-   and the one written in C should be used instead.
-   Define HAVE_ALLOCA to say that the system provides a properly
-   working alloca function and it should be used.
-   Define neither one if an assembler-language alloca
-   in the file alloca.s should be used.  */
-
 #ifdef __GNUC__
-#define HAVE_ALLOCA
-#define alloca __builtin_alloca
 #define C_OPTIMIZE_SWITCH "-O -fstrength-reduce -fomit-frame-pointer"
 #define LIB_STANDARD "-lc /lib/crtn.o"
 #else
 /* This section is correct if you do *not* change src/ymakefile so that
    CFLAGS includes C_OPTIMIZE_SWITCH rather than C_DEBUG_SWITCH.  */
-#define HAVE_ALLOCA
-#define "C_DEBUG_SWITCH -g -O0"
+#define C_DEBUG_SWITCH "-g -O0"
 #define LIB_STANDARD "-lc -lPW /lib/crtn.o"
 /* This section is correct if you do enable C_OPTIMIZE_SWITCH.  */
-/* #define C_ALLOCA */
-/* #define STACK_DIRECTION -1 */
 /* #define C_OPTIMIZE_SWITCH -O2 */
 /* #define LIB_STANDARD -lc /lib/crtn.o */
 #endif
