@@ -403,11 +403,7 @@ mc_allocator_globals_type mc_allocator_globals;
 
 
 /* Number of mark bits: minimum 1, maximum 8. */
-#ifdef NEW_GC
 #define N_MARK_BITS 2
-#else /* not NEW_GC */
-#define N_MARK_BITS 1
-#endif /* not NEW_GC */
 
 
 
@@ -415,6 +411,8 @@ mc_allocator_globals_type mc_allocator_globals;
 /*                           MC Allocator                               */
 /************************************************************************/
 
+/* Set to 1 if memory becomes short. */
+EMACS_INT memory_shortage;
 
 /*--- misc functions ---------------------------------------------------*/
 
@@ -434,10 +432,11 @@ deadbeef_memory (void *ptr, Bytecount size)
 
 /* Visits all pages (page_headers) hooked into the used heap pages
    list and executes f with the current page header as
-   argument. Needed for sweep. */
-static void
-visit_all_used_page_headers (void (*f) (page_header *ph))
+   argument. Needed for sweep.  Returns number of processed pages. */
+static EMACS_INT
+visit_all_used_page_headers (EMACS_INT (*f) (page_header *ph))
 {
+  EMACS_INT number_of_pages_processed = 0;
   EMACS_INT i;
   for (i = 0; i < N_USED_PAGE_LISTS; i++)
     if (PLH_FIRST (USED_HEAP_PAGES (i)))
@@ -446,11 +445,12 @@ visit_all_used_page_headers (void (*f) (page_header *ph))
         while (PH_NEXT (ph))
           {
             page_header *next = PH_NEXT (ph); /* in case f removes the page */
-            f (ph);
+            number_of_pages_processed += f (ph);
             ph = next;
           }
-        f (ph);
+        number_of_pages_processed += f (ph);
       }
+  return number_of_pages_processed;
 }
 
 
@@ -773,11 +773,9 @@ set_mark_bit (void *ptr, EMACS_INT value)
   assert (ph && PH_ON_USED_LIST_P (ph));
   if (ph)
     {
-#ifdef NEW_GC
       if (value == BLACK)
 	if (!PH_BLACK_BIT (ph))
 	  PH_BLACK_BIT (ph) = 1;
-#endif /* NEW_GC */
       SET_BIT (ph, get_mark_bit_index (ptr, ph), value);
     }
 }
@@ -1140,9 +1138,12 @@ expand_heap (EMACS_INT needed_pages)
   void *real_start;
 
   /* determine number of pages the heap should grow */
-  n_pages = needed_pages + (HEAP_SIZE / (PAGE_SIZE * HEAP_GROWTH_DIVISOR));
-  if (n_pages < MIN_HEAP_INCREASE)
-    n_pages = MIN_HEAP_INCREASE;
+  if (memory_shortage)
+    n_pages = needed_pages;
+  else
+    n_pages = max (MIN_HEAP_INCREASE, 
+		   needed_pages
+		   + (HEAP_SIZE / (PAGE_SIZE * HEAP_GROWTH_DIVISOR)));
 
   /* get the real values */
   real_size = (n_pages * PAGE_SIZE) + PAGE_SIZE;
@@ -1271,10 +1272,8 @@ remove_page_from_used_list (page_header *ph)
 {
   page_list_header *plh = PH_PLH (ph);
 
-#ifdef NEW_GC
   if (gc_in_progress && PH_PROTECTION_BIT (ph)) ABORT();
   /* cleanup: remove memory protection, zero page_header bits. */
-#endif /* not NEW_GC */
 
 #ifdef MEMORY_USAGE_STATS
   PLH_TOTAL_CELLS (plh) -= PH_CELLS_ON_PAGE (ph);
@@ -1482,67 +1481,20 @@ mark_free_list (page_header *ph)
   free_link *fl = PH_FREE_LIST (ph);
   while (fl)
     {
-#ifdef NEW_GC
       SET_BIT (ph, get_mark_bit_index (fl, ph), BLACK);
-#else /* not NEW_GC */
-      SET_BIT (ph, get_mark_bit_index (fl, ph), 1);
-#endif /* not NEW_GC */
       fl = NEXT_FREE (fl);
     }
 }
 
-/* Finalize a page. You have to tell mc-alloc how to call your
-   object's finalizer. Therefore, you have to define the macro
-   MC_ALLOC_CALL_FINALIZER(ptr). This macro should do nothing else
-   then test if there is a finalizer and call it on the given
-   argument, which is the heap address of the object. */
-static void
-finalize_page (page_header *ph)
-{
-  EMACS_INT heap_space = (EMACS_INT) PH_HEAP_SPACE (ph);
-  EMACS_INT heap_space_step = PH_CELL_SIZE (ph);
-  EMACS_INT mark_bit = 0;
-  EMACS_INT mark_bit_max_index = PH_CELLS_ON_PAGE (ph);
-  unsigned int bit = 0;
-
-  mark_free_list (ph);
-
-#ifdef NEW_GC
-  /* ARRAY_BIT_HACK */
-  if (PH_ARRAY_BIT (ph))
-    for (mark_bit = 0; mark_bit < mark_bit_max_index; mark_bit++)
-      {
-	GET_BIT (bit, ph, mark_bit * N_MARK_BITS);
-	if (bit)
-	  {
-	    return;
-	  }
-      }
-#endif /* NEW_GC */
-
-  for (mark_bit = 0; mark_bit < mark_bit_max_index; mark_bit++)
-    {
-      GET_BIT (bit, ph, mark_bit * N_MARK_BITS);
-#ifdef NEW_GC
-      if (bit == WHITE) 
-#else /* not NEW_GC */
-      if (bit == 0) 
-#endif /* not NEW_GC */
-        {
-	  EMACS_INT ptr = (heap_space + (heap_space_step * mark_bit));
-	  MC_ALLOC_CALL_FINALIZER ((void *) ptr);
-        }
-    }
-}
-
 
 /* Finalize a page for disksave. XEmacs calls this routine before it
    dumps the heap image. You have to tell mc-alloc how to call your
    object's finalizer for disksave. Therefore, you have to define the
    macro MC_ALLOC_CALL_FINALIZER_FOR_DISKSAVE(ptr). This macro should
    do nothing else then test if there is a finalizer and call it on
-   the given argument, which is the heap address of the object. */
-static void
+   the given argument, which is the heap address of the object.
+   Returns number of processed pages. */
+static EMACS_INT
 finalize_page_for_disksave (page_header *ph)
 {
   EMACS_INT heap_space = (EMACS_INT) PH_HEAP_SPACE (ph);
@@ -1555,29 +1507,24 @@ finalize_page_for_disksave (page_header *ph)
       EMACS_INT ptr = (heap_space + (heap_space_step * mark_bit));
       MC_ALLOC_CALL_FINALIZER_FOR_DISKSAVE ((void *) ptr);
     }
+  return 1;
 }
 
 
-/* Finalizes the heap. */
-void
-mc_finalize (void)
-{
-  visit_all_used_page_headers (finalize_page);
-}
-
-
-/* Finalizes the heap for disksave. */
-void
+/* Finalizes the heap for disksave.  Returns number of processed
+   pages. */
+EMACS_INT
 mc_finalize_for_disksave (void)
 {
-  visit_all_used_page_headers (finalize_page_for_disksave);
+  return visit_all_used_page_headers (finalize_page_for_disksave);
 }
 
 
-/* Sweeps a page: all the non-marked cells are freed. If the page is empty
-   in the end, it is removed. If some cells are free, it is moved to the
-   front of its page header list. Full pages stay where they are. */
-static void
+/* Sweeps a page: all the non-marked cells are freed. If the page is
+   empty in the end, it is removed. If some cells are free, it is
+   moved to the front of its page header list. Full pages stay where
+   they are.  Returns number of processed pages.*/
+static EMACS_INT
 sweep_page (page_header *ph)
 {
   Rawbyte *heap_space = (Rawbyte *) PH_HEAP_SPACE (ph);
@@ -1588,7 +1535,6 @@ sweep_page (page_header *ph)
 
   mark_free_list (ph);
 
-#ifdef NEW_GC
   /* ARRAY_BIT_HACK */
   if (PH_ARRAY_BIT (ph))
     for (mark_bit = 0; mark_bit < mark_bit_max_index; mark_bit++)
@@ -1598,23 +1544,16 @@ sweep_page (page_header *ph)
 	  {
 	    zero_mark_bits (ph);
 	    PH_BLACK_BIT (ph) = 0;
-	    return;
+	    return 1;
 	  }
       }
-#endif /* NEW_GC */
 
   for (mark_bit = 0; mark_bit < mark_bit_max_index; mark_bit++)
     {
       GET_BIT (bit, ph, mark_bit * N_MARK_BITS);
-#ifdef NEW_GC
       if (bit == WHITE) 
-#else /* not NEW_GC */
-      if (bit == 0) 
-#endif /* not NEW_GC */
 	{
-#ifdef NEW_GC
 	  GC_STAT_FREED;
-#endif /* NEW_GC */
 	  remove_cell (heap_space + (heap_space_step * mark_bit), ph);
 	}
     }
@@ -1624,45 +1563,25 @@ sweep_page (page_header *ph)
     remove_page_from_used_list (ph);
   else if (PH_CELLS_USED (ph) < PH_CELLS_ON_PAGE (ph))
     move_page_header_to_front (ph);
+
+  return 1;
 }
 
 
-/* Sweeps the heap. */
-void
+/* Sweeps the heap.  Returns number of processed pages. */
+EMACS_INT
 mc_sweep (void)
 {
-  visit_all_used_page_headers (sweep_page);
+  return visit_all_used_page_headers (sweep_page);
 }
 
 
 /* Frees the cell pointed to by ptr. */
 void
-mc_free (void *ptr)
+mc_free (void *UNUSED (ptr))
 {
-  page_header *ph;
-
-#ifdef NEW_GC
-  /* Do not allow manual freeing while a gc is running. Data is going
-     to be freed next gc cycle. */
-  if (write_barrier_enabled || gc_in_progress)
-    return;
-#endif /* NEW_GC */
-
-  ph = get_page_header (ptr);
-  assert (ph);
-  assert (PH_PLH (ph));
-  assert (PLH_LIST_TYPE (PH_PLH (ph)) != FREE_LIST);
-
-#ifdef NEW_GC
-  if (PH_ON_USED_LIST_P (ph))
-    SET_BIT (ph, get_mark_bit_index (ptr, ph), WHITE);
-#endif /* NEW_GC */
-  remove_cell (ptr, ph);
-
-  if (PH_CELLS_USED (ph) == 0)
-    remove_page_from_used_list (ph);
-  else if (PH_CELLS_USED (ph) < PH_CELLS_ON_PAGE (ph))
-    move_page_header_to_front (ph);
+  /* Manual frees are not allowed with asynchronous finalization */
+  return;
 }
 
 
@@ -1871,7 +1790,6 @@ syms_of_mc_alloc (void)
 }
 
 
-#ifdef NEW_GC
 /*--- incremental garbage collector ----------------------------------*/
 
 /* access dirty bit of page header */
@@ -1955,8 +1873,8 @@ fault_on_protected_page (void *ptr)
 
 
 /* Protect the heap page of given page header ph if black objects are
-   on the page. */
-static void
+   on the page.  Returns number of processed pages. */
+static EMACS_INT
 protect_heap_page (page_header *ph)
 {
   if (PH_BLACK_BIT (ph))
@@ -1965,20 +1883,23 @@ protect_heap_page (page_header *ph)
       EMACS_INT heap_space_size = PH_N_PAGES (ph) * PAGE_SIZE;
       vdb_protect ((void *) heap_space, heap_space_size);
       PH_PROTECTION_BIT (ph) = 1;
+      return 1;
     }
+  return 0;
 }
 
-/* Protect all heap pages with black objects. */
-void
+/* Protect all heap pages with black objects.  Returns number of
+   processed pages.*/
+EMACS_INT
 protect_heap_pages (void)
 {
-  visit_all_used_page_headers (protect_heap_page);
+  return visit_all_used_page_headers (protect_heap_page);
 }
 
 
 /* Remove protection (if there) of heap page of given page header
-   ph. */
-static void
+   ph.  Returns number of processed pages. */
+static EMACS_INT
 unprotect_heap_page (page_header *ph)
 {
   if (PH_PROTECTION_BIT (ph))
@@ -1987,14 +1908,17 @@ unprotect_heap_page (page_header *ph)
       EMACS_INT heap_space_size = PH_N_PAGES (ph) * PAGE_SIZE;
       vdb_unprotect (heap_space, heap_space_size);
       PH_PROTECTION_BIT (ph) = 0;
+      return 1;
     }
+  return 0;
 }
 
-/* Remove protection for all heap pages which are protected. */
-void
+/* Remove protection for all heap pages which are protected.  Returns
+   number of processed pages. */
+EMACS_INT
 unprotect_heap_pages (void)
 {
-  visit_all_used_page_headers (unprotect_heap_page);
+  return visit_all_used_page_headers (unprotect_heap_page);
 }
 
 /* Remove protection and mark page dirty. */
@@ -2062,5 +1986,3 @@ object_on_heap_p (void *ptr)
   page_header *ph = get_page_header_internal (ptr);
   return (ph && PH_ON_USED_LIST_P (ph));
 }
-
-#endif /* NEW_GC */
