@@ -2167,7 +2167,7 @@ mswindows_font_spec_matches_charset (struct device *d, Lisp_Object charset,
 				     const Ibyte *nonreloc,
 				     Lisp_Object reloc,
 				     Bytecount offset, Bytecount length,
-				     int stage)
+				     enum font_specifier_matchspec_stages stage)
 {
   return stage ?
      mswindows_font_spec_matches_charset_stage_2 (d, charset, nonreloc,
@@ -2182,7 +2182,8 @@ mswindows_font_spec_matches_charset (struct device *d, Lisp_Object charset,
 
 static Lisp_Object
 mswindows_find_charset_font (Lisp_Object device, Lisp_Object font,
-			     Lisp_Object charset, int stage)
+			     Lisp_Object charset,
+			     enum font_specifier_matchspec_stages stage)
 {
   Lisp_Object fontlist, fonttail;
 
