@@ -72,19 +72,21 @@ x_inhibit_scrollbar_slider_size_change (void)
 static void
 x_free_scrollbar_instance (struct scrollbar_instance *instance)
 {
-  if (SCROLLBAR_X_NAME (instance))
-    xfree (SCROLLBAR_X_NAME (instance), char *);
-
-  if (SCROLLBAR_X_WIDGET (instance))
-    {
-      if (XtIsManaged (SCROLLBAR_X_WIDGET (instance)))
-	XtUnmanageChild (SCROLLBAR_X_WIDGET (instance));
-
-      lw_destroy_all_widgets (SCROLLBAR_X_ID (instance));
-    }
-
   if (instance->scrollbar_data)
-    xfree (instance->scrollbar_data, void *);
+    {
+      if (SCROLLBAR_X_NAME (instance))
+	xfree (SCROLLBAR_X_NAME (instance), char *);
+
+      if (SCROLLBAR_X_WIDGET (instance))
+	{
+	  if (XtIsManaged (SCROLLBAR_X_WIDGET (instance)))
+	    XtUnmanageChild (SCROLLBAR_X_WIDGET (instance));
+
+	  lw_destroy_all_widgets (SCROLLBAR_X_ID (instance));
+	}
+
+      xfree (instance->scrollbar_data, void *);
+    }
 }
 
 /* A device method. */
@@ -703,14 +705,9 @@ x_compute_scrollbar_instance_usage (struct device *UNUSED (d),
       struct x_scrollbar_data *data =
 	(struct x_scrollbar_data *) inst->scrollbar_data;
 
-#ifdef MC_ALLOC
-      total += mc_alloced_storage_size (sizeof (*data), ovstats);
-      total += mc_alloced_storage_size (1 + strlen (data->name), ovstats);
-#else /* not MC_ALLOC */
       total += malloced_storage_size (data, sizeof (*data), ovstats);
       total += malloced_storage_size (data->name, 1 + strlen (data->name),
 				      ovstats);
-#endif /* not MC_ALLOC */
       inst = inst->next;
     }
 
