@@ -33,6 +33,8 @@
 ;; This file will be (read)ed by a non-mule XEmacs, so don't use
 ;; literal non-Latin1 characters.  Use (make-char) instead.
 
+(require 'bytecomp)
+
 ;;-----------------------------------------------------------------
 ;; Test whether all legal chars may be safely inserted to a buffer.
 ;;-----------------------------------------------------------------
@@ -78,6 +80,38 @@ the Assert macro checks for correctness."
 ;; once compiled, for no good reason.
 (test-chars t)
 
+(defun unicode-code-point-to-utf-8-string (code-point)
+  "Convert a Unicode code point to the equivalent UTF-8 string. 
+This is a naive implementation in Lisp.  "
+  (check-argument-type 'natnump code-point)
+  (check-argument-range code-point 0 #x1fffff)
+  (if (< code-point #x80)
+      (format "%c" code-point)
+    (if (< code-point #x800)
+	(format "%c%c" 
+		;; ochars[0] = 0xC0 | (input & ~(0xFFFFF83F)) >> 6;
+		(logior #xc0 (lsh (logand code-point #x7c0) -6))
+		;; ochars[1] = 0x80 | input & ~(0xFFFFFFC0);
+		(logior #x80 (logand code-point #x3f)))
+      (if (< code-point #x00010000)
+	  (format "%c%c%c" 
+		  ;; ochars[0] = 0xE0 | (input >> 12) & ~(0xFFFFFFF0); 
+		  (logior #xe0 (logand (lsh code-point -12) #x0f))
+		  ;; ochars[1] = 0x80 | (input >> 6) & ~(0xFFFFFFC0); 
+		  (logior #x80 (logand (lsh code-point -6) #x3f))
+		  ;; ochars[2] = 0x80 | input & ~(0xFFFFFFC0); 
+		  (logior #x80 (logand code-point #x3f)))
+	(if (< code-point #x200000)
+	    (format "%c%c%c%c" 
+		    ;; ochars[0] = 0xF0 | (input >> 18) & ~(0xFFFFFFF8)
+		    (logior #xF0 (logand (lsh code-point -18) #x7))
+		    ;; ochars[1] = 0x80 | (input >> 12) & ~(0xFFFFFFC0);
+		    (logior #x80 (logand (lsh code-point -12) #x3f))
+		    ;; ochars[2] = 0x80 | (input >> 6) & ~(0xFFFFFFC0); 
+		    (logior #x80 (logand (lsh code-point -6) #x3f))
+		    ;; ochars[3] = 0x80 | input & ~(0xFFFFFFC0); 
+		    (logior #x80 (logand code-point #x3f))))))))
+
 ;;-----------------------------------------------------------------
 ;; Test string modification functions that modify the length of a char.
 ;;-----------------------------------------------------------------
@@ -293,33 +327,35 @@ the Assert macro checks for correctness."
   ;;---------------------------------------------------------------
   ;; Test file-system character conversion (and, en passant, file ops)
   ;;---------------------------------------------------------------
-  (let* ((scaron (make-char 'latin-iso8859-2 57))
-	 (latin2-string (make-string 4 scaron))
+  (let* ((dstroke (make-char 'latin-iso8859-2 80))
+	 (latin2-string (make-string 4 dstroke))
 	 (prefix (concat (file-name-as-directory
 			  (file-truename (temp-directory)))
 			 latin2-string))
-	 (name1 (make-temp-name prefix))
-	 (name2 (make-temp-name prefix))
 	 (file-name-coding-system
 	  ;; 'iso-8859-X doesn't work on darwin (as of "Panther" 10.3), it
 	  ;; seems to know that file-name-coding-system is definitely utf-8
 	  (if (string-match "darwin" system-configuration)
 	      'utf-8
 	    'iso-8859-2))
-	 )
+         ;; make-temp-name does stat(), which on OS X requires that you
+         ;; normalise, where open() will normalise for you. Previously we
+         ;; used scaron as the Latin-2 character, and make-temp-name errored
+         ;; on OS X. LATIN CAPITAL LETTER D WITH STROKE does decompose.
+         (name1 (make-temp-name prefix))
+         (name2 (make-temp-name prefix)))
     ;; This is how you suppress output from `message', called by `write-region'
-    (flet ((append-message (&rest args) ()))
-      (Assert (not (equal name1 name2)))
-      (Assert (not (file-exists-p name1)))
-      (write-region (point-min) (point-max) name1)
-      (Assert (file-exists-p name1))
-      (when (fboundp 'make-symbolic-link)
-	(make-symbolic-link name1 name2)
-	(Assert (file-exists-p name2))
-	(Assert (equal (file-truename name2) name1))
-	(Assert (equal (file-truename name1) name1)))
-
-      (ignore-file-errors (delete-file name1) (delete-file name2))))
+    (Assert (not (equal name1 name2)))
+    (Assert (not (file-exists-p name1)))
+    (Silence-Message
+     (write-region (point-min) (point-max) name1))
+    (Assert (file-exists-p name1))
+    (when (fboundp 'make-symbolic-link)
+      (make-symbolic-link name1 name2)
+      (Assert (file-exists-p name2))
+      (Assert (equal (file-truename name2) name1))
+      (Assert (equal (file-truename name1) name1)))
+    (ignore-file-errors (delete-file name1) (delete-file name2)))
 
   ;; Add many more file operation tests here...
 
@@ -336,27 +372,224 @@ the Assert macro checks for correctness."
   
     (Check-Error wrong-type-argument (set-unicode-conversion scaron -10000)))
 
+  (dolist (utf-8-char 
+	   '("\xc6\x92"		  ;; U+0192 LATIN SMALL LETTER F WITH HOOK
+	     "\xe2\x81\x8a"	  ;; U+204A TIRONIAN SIGN ET
+	     "\xe2\x82\xae"	  ;; U+20AE TUGRIK SIGN
+	     "\xf0\x9d\x92\xbd"	  ;; U+1D4BD MATHEMATICAL SCRIPT SMALL H
+	     "\xf0\x9d\x96\x93"   ;; U+1D593 MATHEMATICAL BOLD FRAKTUR SMALL N
+	     "\xf0\xaf\xa8\x88"   ;; U+2FA08 CJK COMPATIBILITY FOR U+4BCE
+	     "\xf4\x8f\xbf\xbd")) ;; U+10FFFD <Plane 16 Private Use, Last>
+    (let* ((xemacs-character (car (append 
+				  (decode-coding-string utf-8-char 'utf-8) 
+				  nil)))
+	   (xemacs-charset (car (split-char xemacs-character))))
+
+      ;; Trivial test of the UTF-8 support of the escape-quoted character set. 
+      (Assert (equal (decode-coding-string utf-8-char 'utf-8)
+		     (decode-coding-string (concat "\033%G" utf-8-char)
+					   'escape-quoted)))
+
+      ;; Check that the reverse mapping holds. 
+      (Assert (equal (unicode-code-point-to-utf-8-string 
+		      (encode-char xemacs-character 'ucs))
+		     utf-8-char))
+
+      ;; Check that, if this character has been JIT-allocated, it is encoded
+      ;; in escape-quoted using the corresponding UTF-8 escape. 
+      (when (charset-property xemacs-charset 'encode-as-utf-8)
+	(Assert (equal (concat "\033%G" utf-8-char)
+		       (encode-coding-string xemacs-character 'escape-quoted)))
+	(Assert (equal (concat "\033%G" utf-8-char)
+		       (encode-coding-string xemacs-character 'ctext))))))
+
+  (loop
+    for (code-point encoded) 
+    in '((#x10000 "\xd8\x00\xdc\x00")
+         (#x10FFFD "\xdb\xff\xdf\xfd"))
+    do (Assert (equal (encode-coding-string 
+                       (decode-char 'ucs code-point) 'utf-16)
+                      encoded)))
+         
+  ;;---------------------------------------------------------------
+  ;; Regression test for a couple of CCL-related bugs. 
+  ;;---------------------------------------------------------------
+
+  (let ((ccl-vector [0 0 0 0 0 0 0 0 0]))
+    (define-ccl-program ccl-write-two-control-1-chars 
+      `(1 
+	((r0 = ,(charset-id 'control-1))
+	 (r1 = 0) 
+	 (write-multibyte-character r0 r1) 
+	 (r1 = 31) 
+	 (write-multibyte-character r0 r1))) 
+      "CCL program that writes two control-1 multibyte characters.") 
+ 
+    (Assert (equal 
+	     (ccl-execute-on-string 'ccl-write-two-control-1-chars  
+				    ccl-vector "") 
+	     (format "%c%c" (make-char 'control-1 0) 
+		     (make-char 'control-1 31))))
+
+    (define-ccl-program ccl-unicode-two-control-1-chars 
+      `(1 
+	((r0 = ,(charset-id 'control-1))
+	 (r1 = 31) 
+	 (mule-to-unicode r0 r1) 
+	 (r4 = r0) 
+	 (r3 = ,(charset-id 'control-1))
+	 (r2 = 0) 
+	 (mule-to-unicode r3 r2))) 
+      "CCL program that writes two control-1 UCS code points in r3 and r4")
+
+    ;; Re-initialise the vector, mainly to clear the instruction counter,
+    ;; which is its last element.
+    (setq ccl-vector [0 0 0 0 0 0 0 0 0])
+ 
+    (ccl-execute-on-string 'ccl-unicode-two-control-1-chars ccl-vector "") 
+ 
+    (Assert (and (eq (aref ccl-vector 3)  
+                   (encode-char (make-char 'control-1 0) 'ucs)) 
+               (eq (aref ccl-vector 4)  
+                   (encode-char (make-char 'control-1 31) 'ucs)))))
+
   ;;---------------------------------------------------------------
   ;; Test charset-in-* functions
   ;;---------------------------------------------------------------
   (with-temp-buffer
     (insert-file-contents (locate-data-file "HELLO"))
-    ;; #### rewrite robustly, both assume that the tested implementation
-    ;; uses the same algorithm as was used by the version current at time
-    ;; this test was written
-    (Assert (equal (charsets-in-region (point-min) (point-max))
-		   '(korean-ksc5601 chinese-big5-1 chinese-gb2312
-		     japanese-jisx0212 katakana-jisx0201 japanese-jisx0208
-		     vietnamese-viscii-lower thai-xtis cyrillic-iso8859-5
-		     hebrew-iso8859-8 greek-iso8859-7 latin-iso8859-1
-		     latin-iso8859-2 arabic-2-column arabic-1-column
-		     ethiopic ascii)))
-    (Assert (equal (charsets-in-string (buffer-substring (point-min)
+    (Assert (equal 
+             ;; The sort is to make the algorithm of charsets-in-region
+             ;; irrelevant.
+             (sort (charsets-in-region (point-min) (point-max))
+                   'string<)
+             '(arabic-1-column arabic-2-column ascii chinese-big5-1
+               chinese-gb2312 cyrillic-iso8859-5 ethiopic greek-iso8859-7
+               hebrew-iso8859-8 japanese-jisx0208 japanese-jisx0212
+               katakana-jisx0201 korean-ksc5601 latin-iso8859-1
+               latin-iso8859-2 thai-xtis vietnamese-viscii-lower)))
+    (Assert (equal 
+             (sort (charsets-in-string (buffer-substring (point-min)
 							 (point-max)))
-		   '(korean-ksc5601 chinese-big5-1 chinese-gb2312
-		     japanese-jisx0212 katakana-jisx0201 japanese-jisx0208
-		     vietnamese-viscii-lower thai-xtis cyrillic-iso8859-5
-		     hebrew-iso8859-8 greek-iso8859-7 latin-iso8859-1
-		     latin-iso8859-2 arabic-2-column arabic-1-column
-		     ethiopic ascii))))
+                   'string<)
+             '(arabic-1-column arabic-2-column ascii chinese-big5-1
+               chinese-gb2312 cyrillic-iso8859-5 ethiopic greek-iso8859-7
+               hebrew-iso8859-8 japanese-jisx0208 japanese-jisx0212
+               katakana-jisx0201 korean-ksc5601 latin-iso8859-1
+               latin-iso8859-2 thai-xtis vietnamese-viscii-lower))))
+
+  ;; Language environments. 
+  (dolist (language (mapcar 'car language-info-alist))
+    (set-language-environment language)
+    (Assert (equal language current-language-environment))
+    (set-input-method (get-language-info language 'input-method))
+    (Assert (equal (get-language-info language 'input-method) 
+                   current-input-method))
+    (dolist (charset (get-language-info language 'charset))
+      (Assert (charsetp (find-charset charset))))
+    (dolist (coding-system (get-language-info language 'coding-system))
+      (Assert (coding-system-p (find-coding-system coding-system))))
+    (dolist (coding-system (get-language-info language 'coding-system))
+      (Assert (coding-system-p (find-coding-system coding-system)))))
+
+  (with-temp-buffer
+    (flet
+        ((Assert-elc-is-escape-quoted ()
+           "Assert the current buffer has an escape-quoted cookie if compiled."
+           (save-excursion
+             (let ((byte-compile-result (byte-compile-from-buffer
+                                         (current-buffer) nil nil))
+                   (temporary-file-name (make-temp-name
+                                         (expand-file-name "zjPQ2Pk"
+                                                           (temp-directory)))))
+               (byte-compile-insert-header
+                temporary-file-name
+                (current-buffer)
+                byte-compile-result)
+               (Assert (string-match "^;;;###coding system: escape-quoted"
+                                     (buffer-substring nil nil
+                                                       byte-compile-result))))))
+         (Assert-elc-has-no-specified-encoding ()
+           "Assert the current buffer has no coding cookie if compiled."
+           (save-excursion
+             (let ((byte-compile-result (byte-compile-from-buffer
+                                         (current-buffer) nil nil))
+                   (temporary-file-name (make-temp-name
+                                         (expand-file-name "zjPQ2Pk"
+                                                           (temp-directory)))))
+               (byte-compile-insert-header
+                temporary-file-name
+                (current-buffer)
+                byte-compile-result)
+               (Assert (not (string-match
+                             ";;;###coding system:"
+                             (buffer-substring nil nil byte-compile-result))))))))
+      (insert 
+       ;; Create a buffer creating the Unicode escapes. 
+       #r" (defvar testing-mule-compilation-handling 
+            (string ?\u371E   ;; kDefinition beautiful; pretty, used 
+                              ;; in girl's name
+                ?\U0002A6A9   ;; kDefinition	(Cant.) sound of shouting
+                ?\U0002A65B   ;; kDefinition	(Cant.) decayed teeth; 
+                              ;; tongue-tied
+                ?\U00010400   ;; DESERET CAPITAL LETTER LONG I
+                    ?\u3263)) ;; CIRCLED HANGUL RIEUL ")
+
+      (Assert-elc-is-escape-quoted)
+      (delete-region (point-min) (point-max))
+
+      (insert
+       ;; This time, the buffer will contain the actual characters, because of
+       ;; u flag to the #r. 
+       #ru" (defvar testing-mule-compilation-handling 
+            (string ?\u371E   ;; kDefinition beautiful; pretty, used 
+                              ;; in girl's name
+                ?\U0002A6A9   ;; kDefinition	(Cant.) sound of shouting
+                ?\U0002A65B   ;; kDefinition	(Cant.) decayed teeth; 
+                              ;; tongue-tied
+                ?\U00010400   ;; DESERET CAPITAL LETTER LONG I
+                    ?\u3263)) ;; CIRCLED HANGUL RIEUL ")
+    
+      (Assert-elc-is-escape-quoted)
+      (delete-region (point-min) (point-max))
+
+      (insert
+       ;; Just a single four character escape. 
+       #r" (defvar testing-mule-compilation-handling 
+            (string ?\u371E))   ;; kDefinition beautiful; pretty, used")
+
+      (Assert-elc-is-escape-quoted)
+      (delete-region (point-min) (point-max))
+
+      (insert
+       ;; Just a single eight character escape. 
+       #r" (defvar testing-mule-compilation-handling 
+            (string ?\U0002A65B))   ;; kDefinition (Cant.) decayed teeth;")
+
+      (Assert-elc-is-escape-quoted)
+      (delete-region (point-min) (point-max))
+
+      (insert
+       ;; A single latin-1 hex digit escape
+       #r" (defvar testing-mule-compilation-handling 
+            (string ?\xab))   ;; LEFT-POINTING DOUBLE ANGLE QUOTATION MARK")
+      
+      (Assert-elc-has-no-specified-encoding)
+      (delete-region (point-min) (point-max))
+
+      (insert
+       ;; A single latin-1 character
+       #ru" (defvar testing-mule-compilation-handling 
+            (string ?\u00AB))   ;; LEFT-POINTING DOUBLE ANGLE QUOTATION MARK")
+      
+      (Assert-elc-has-no-specified-encoding)
+      (delete-region (point-min) (point-max))
+
+      (insert
+       ;; Just ASCII. 
+       #r" (defvar testing-mule-compilation-handling 
+            (string ?A))   ;; LATIN CAPITAL LETTER A")
+      
+      (Assert-elc-has-no-specified-encoding)
+      (delete-region (point-min) (point-max))))
   )
