@@ -2,11 +2,11 @@
 emacs_is_beta=t
 emacs_major_version=21
 emacs_minor_version=5
-emacs_beta_version=27
-xemacs_codename="fiddleheads"
+emacs_beta_version=28
+xemacs_codename="fuki"
 emacs_kit_version=
 infodock_major_version=4
 infodock_minor_version=0
 infodock_build_version=8
 xemacs_extra_name=""
-xemacs_release_date="2006-05-16"
+xemacs_release_date="2007-05-21"
